#!/usr/local/bin/perl -w
# -*- perl -*-

# Copyright (c) 2003 by Jeremy Kister
# Author: Jeremy Kister <argus-devel at jeremykister.com>
# Date: 2003-Jun-20 04:31 (EDT)
# Function: send messages via AIM/otc
#
# take a look at http://www.jeremykister.com
# for hosting/colo/Internet stuff, check out http://www.nntx.net
#
# some code plucked from Net::AIM's author's example code (who also stole it) 

# start this before argus is started; just as qpage and/or sendmail are.
# I recommend running this under DJB's daemontools (http://cr.yp.to/daemontools.html)
# careful to only use aimc as the user aim.pl is running as or aim.pl wont have
# permission to remove the aimc tmp file. 

# AIM sends some weird characters sometimes, yielding "PARSE: how many args in '2'?"
# You can hush this by commenting the line in lib/perl5/site_perl/5.x.x/Net/AIM/Connection.pm
# there is a number 2 being sent to us, as well as 3 null characters 

# go to www.aim.com and sign up for a new username/pw if you need one.
# note: as of 2003/06/20, there is no need to even read their confirmation email

use Net::AIM;
use IO::Handle;
use Getopt::Std;

getopts('u:g:b:s:p:D');

# change uid/gid code *STOLEN* from Jeff Weisberg (argusd, http://argus.tcp4me.com)
# Change nobody if you'd like, or to a user who has argusctl permissions.
# change uid/gid
if( $opt_g ){
    my $gid = (getgrnam($opt_g))[2];
    $gid = $opt_g if( !defined($gid) && $opt_g =~ /^\d+$/ );
    die "invalid group for -g option. aborting.\n" unless defined $gid;
    $( = $gid; $) = $gid;
}
if( $opt_u ){
    my $uid = (getpwnam($opt_u))[2];
    $uid = $opt_u if( !defined($uid) && $opt_u =~ /^\d+$/ );
    die "invalid user for -u option. aborting.\n" unless defined $uid;
    $! = 0;
    $< = $> = $uid;
    die "unable to change uid: $!\n" if $!;
}

unless( defined($opt_s) && defined($opt_p) ){
	print "syntax:\n",
	      "	aim.pl [-u <username>] [-g <group>] [-D] [-b </path/argus/sbin>] -s <screenname> -p <password>\n",
	      "	options:\n",
	      "		-u	username to run as\n",
 	      "		-g	group to run as\n",
	      "		-D	send debugging info to STDERR\n",
	      "		-b	alternative path to argus sbin directory (default: /usr/local/sbin)\n",
	      "		-s	AIM screenname\n",
	      "		-p	AIM password\n\n";
	exit;
}

$sbin="/usr/local/sbin" unless($sbin=$opt_b);

print STDERR "starting up...\n" if($opt_D);

# Create a new Net-AIM instance.
my $aim = new Net::AIM;
$aim->newconn( Screenname => $opt_s,
               Password   => $opt_p )
                or die "Error: Cannot connect to the AIM server!";
$conn = $aim->getconn();

print STDERR "connected to aim server as '$opt_s'.\n" if($opt_D);

# Set up the handlers for commands issued by the server.
$conn->set_handler('im_in', \&on_im);
$conn->set_handler('error', \&on_error);
$conn->set_handler('eviled', \&on_evil);
$conn->set_handler('config', \&on_config);

$msg = $newmsg = $idle_time = 0;

while(1){
	$aim->do_one_loop();

	# check to see if our queue has any msgs in it
	if( open Q, "/tmp/aimqueue" ){
		$sendmsg = undef;
		chomp($recipient=<Q>);
		$recipient =~ s/^recipient:\s+//;
		while(<Q>){
			$sendmsg .= "$_";
		}
		close Q;
		unlink("/tmp/aimqueue") || die "cannot remove /tmp/aimqueue: $!\n";
		$aim->send_im($recipient, $sendmsg);
		$idle_time=0;
	}

	# deal with incoming messages
	if($newmsg == 1){
		print STDERR "'$sender'> \"$msg\"\n" if($opt_D);

		# we can do fun stuff here
		# we will have to worry about evil lusers here - 
		if( $msg =~ /^ACK:\s+(\d+)$/ ){
			$pageid=$1;
			#$send="Acking PageID: $pageid";
			#$send .= `$sbin/argusctl notify_ack idno=$pageid`;
		}elsif( $msg eq "argusctl status" ){
			$send=`$sbin/argusctl status`;
		}
		if(defined($send)){
			$aim->send_im($sender,$send);
			print STDERR "'$opt_s'> \"$send\"\n" if($opt_D);
			$send=undef;
			$idle_time=0; #only reset idle if a valid command was sent to us
		}
		$newmsg=0;
	}

	if($idle_time == 0){
		if(defined($sentaway)){
			print STDERR "WHO!? WHA!? damn it.. i hate being woken up\n" if($opt_D);
			$aim->set_away();
			$sentaway=undef;
		}
		sleep 1;
		$idle_time=1;
	}elsif($idle_time > 60){
		sleep 15;
		$idle_time=($idle_time+15);
		unless(defined($sentaway)){
			if($idle_time > 120){
				print STDERR "<yawn> taking a nap.. ($idle_time seconds)\n" if($opt_D);
				$aim->set_away($idle_time);
				$sentaway=1;
			}
		}
	}else{
		sleep 5;
		$idle_time=($idle_time+5);
	}
}

sub on_config {
	# called when the bot is being loaded.
	my ($self, $evt, $from, $to) = @_;
	
	my $str = shift @{$evt->args()};
	$self->set_config_str($str, 1);

	#Set up my profile.
	$self->set_info("KisterBot v.91b");

	$self->send_config();
}

sub on_error {
	# called when an error occurs while communicating w/ TOC.
	my ($self, $evt) = @_;
	my ($error, @stuff) = @{$evt->args()};

	#Translate the error number into plain English.
	my $errstr = $evt->trans($error);

	#Filter the error string for the content we need.
	$errstr =~ s/\$(\d+)/$stuff[$1]/ge;

	print "AIM ERROR: $errstr\n";
	print STDERR "AIM ERROR: $errstr\n";

	if($errstr eq "A message has been dropped, you are exceeding the server speed limit"){
		sleep(10);
	}elsif($errstr =~ /Wait\s+(\d+)\s+/){
		$seconds=($1*60);
		sleep($seconds+5);
	}
}

sub on_evil {
	# when the bot recieves a warning.
	my ($self, $evt, $from, $to) = @_;
	my ($level, $bastard) = @{$evt->args};

	print "My warning level is $level %.";
	print STDERR "My warning level is $level %.";

	if($bastard !~ /^\s*$/){
		#Remove spaces from $bastard; make it lowercase.
		$bastard = lc ($bastard);
		$bastard =~ s/ //g;

		#Warn and block him.
		$aim->evil($bastard, 0);
		$aim->add_deny(1, 'Buddies', $bastard);
	}
}

sub on_im {
	# called when the bot recieves an IM.
	($aim, $evt, $from, $to) = @_;
	$args = $evt->args();
	($sender, $friend, $msg) = @$args;

	$sender =~ s/ //g;

	#Format the message without HTML.
	$msg =~ s/<(.|\n)+?>//g;

	STDOUT->autoflush();
	$newmsg = 1;
}
